@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2020 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
@@ -44,13 +44,13 @@
 
    MU_XSCRIPT_NORMAL
      The default level. Everything is logged indiscriminately. Programmers
-     should excercise care when using this mode, as it can reveal security
+     should exercise care when using this mode, as it can reveal security
      sensitive data (such as login credentials, etc.) as well as private
      user's information (mail content).
 
    MU_XSCRIPT_SECURE
      The stream attempts to locate passwords and replaces them with three
-     asteriscs when sending to the log stream. This mode should be used before
+     asterisks when sending to the log stream. This mode should be used before
      the session is authenticated. Currently the following two constructs are
      recognized:
 
@@ -309,14 +309,6 @@ _xscript_read (struct _mu_stream *str, char *buf, size_t bufsize,
   return mu_stream_read (sp->transport, buf, bufsize, pnread);
 }
 
-static int
-_xscript_readdelim (struct _mu_stream *str, char *buf, size_t bufsize,
-		    int delim, size_t *pnread)
-{
-  struct _mu_xscript_stream *sp = (struct _mu_xscript_stream *)str;
-  return mu_stream_readdelim (sp->transport, buf, bufsize, delim, pnread);
-}
-
 static int
 _xscript_write (struct _mu_stream *str, const char *buf, size_t bufsize,
 		  size_t *pnwrite)
@@ -450,19 +442,25 @@ _xscript_ctl (struct _mu_stream *str, int code, int opcode, void *arg)
 	      mu_stream_t *pstr = arg;
 	      mu_stream_t tmp;
 
-	      if (pstr[0] != pstr[1])
+	      if (pstr[1] == NULL)
 		{
-		  status = mu_iostream_create (&tmp, pstr[0], pstr[1]);
-		  if (status)
-		    return status;
+		  tmp = pstr[0];
+		  mu_stream_ref (tmp);
+		  status = 0;
 		}
-	      else
+	      else if (pstr[0] == pstr[1])
 		{
 		  tmp = pstr[0];
 		  mu_stream_ref (tmp);
 		  mu_stream_ref (tmp);
 		  status = 0;
 		}
+	      else
+		{
+		  status = mu_iostream_create (&tmp, pstr[0], pstr[1]);
+		  if (status)
+		    return status;
+		}
 
 	      mu_stream_unref (sp->transport);
 	      sp->transport = tmp;
@@ -582,8 +580,6 @@ mu_xscript_stream_create (mu_stream_t *pref, mu_stream_t transport,
     return ENOMEM;
 
   sp->stream.read = _xscript_read;
-  if (transport->readdelim)
-    sp->stream.readdelim = _xscript_readdelim;
   sp->stream.write = _xscript_write;
   sp->stream.flush = _xscript_flush;
   sp->stream.open = _xscript_open;
