@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2002, 2005, 2007, 2009-2012, 2014-2017 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,6 +22,9 @@
 #ifdef WITH_READLINE
 static char **ml_command_completion (char *cmd, int start, int end);
 static char *ml_command_generator (const char *text, int state);
+static void ml_sig_handler (int);
+#else
+# define ml_sig_handler(n)
 #endif
 
 static volatile int _interrupted;
@@ -33,10 +35,11 @@ sig_handler (int signo)
   switch (signo)
     {
     case SIGINT:
-      if (mailvar_get (NULL, "quit", mailvar_type_boolean, 0) == 0)
+      if (mailvar_is_true (mailvar_name_quit))
 	exit (0);
       _interrupted++;
       break;
+
 #if defined (SIGWINCH)
     case SIGWINCH:
       util_do_command ("set screen=%d", util_getlines ());
@@ -45,19 +48,20 @@ sig_handler (int signo)
       break;
 #endif
     }
+  ml_sig_handler (signo);
 #ifndef HAVE_SIGACTION
   signal (signo, sig_handler);
 #endif
 }
 
 void
-ml_clear_interrupt ()
+ml_clear_interrupt (void)
 {
   _interrupted = 0;
 }
 
 int
-ml_got_interrupt ()
+ml_got_interrupt (void)
 {
   int rc = _interrupted;
   _interrupted = 0;
@@ -86,7 +90,7 @@ ml_getc (FILE *stream)
 }
 
 void
-ml_readline_init ()
+ml_readline_init (void)
 {
   if (!interactive)
     return;
@@ -96,6 +100,7 @@ ml_readline_init ()
   rl_attempted_completion_function =
     (rl_completion_func_t*) ml_command_completion;
   rl_getc_function = ml_getc;
+  rl_catch_signals = 0;
 #endif
 #ifdef HAVE_SIGACTION
   {
@@ -117,7 +122,7 @@ ml_readline_init ()
 }
 
 char *
-ml_readline_internal ()
+ml_readline_internal (void)
 {
   char *buf = NULL;
   size_t size = 0, n;
@@ -135,7 +140,10 @@ ml_readline_internal ()
       return NULL;
     }
   if (n == 0)
-    return NULL;
+    {
+      free (buf);
+      return NULL;
+    }
   mu_rtrim_cset (buf, "\n");
   return buf;
 }
@@ -143,7 +151,7 @@ ml_readline_internal ()
 char *
 ml_readline (const char *prompt)
 {
-  if (interactive)
+  if (interactive) 
     return readline (prompt);
   return ml_readline_internal ();
 }
@@ -159,6 +167,25 @@ ml_readline_with_intr (const char *prompt)
 
 #ifdef WITH_READLINE
 
+/* Readline-specific signal handling */
+static void
+ml_sig_handler (int sig)
+{
+  switch (sig)
+    {
+    case SIGINT:
+      rl_free_line_state ();
+      break;
+
+#if defined (SIGWINCH)
+    case SIGWINCH:
+      rl_resize_terminal ();
+      break;
+#endif
+    }
+  rl_reset_after_signal ();
+}
+
 static char *insert_text;
 
 static int
@@ -224,8 +251,16 @@ ml_command_completion (char *cmd, int start, int end)
       const struct mail_command_entry *entry =
 	mail_find_command (ws.ws_wordv[0]);
       if (entry && entry->command_completion)
-	ret = entry->command_completion (ws.ws_wordc, ws.ws_wordv,
-					 start == end);
+	{
+	  int point = COMPL_DFL;
+
+	  if (start == end)
+	    point |= COMPL_WS;
+	  if (mu_str_skip_class (p + end, MU_CTYPE_SPACE)[0] == 0)
+	    point |= COMPL_LASTARG;
+
+	  ret = entry->command_completion (ws.ws_wordc, ws.ws_wordv, point);
+	}
       else
 	ret = NULL;
     }
@@ -269,165 +304,342 @@ ml_set_completion_append_character (int c)
 }
 
 void
-ml_attempted_completion_over ()
+ml_attempted_completion_over (void)
 {
   rl_attempted_completion_over = 1;
 }
 
+
+/* Concatenate results of two completions. Both A and B are expected to
+   be returned by rl_completion_matches, i.e. their first entry is the
+   longest common prefix of the remaining entries, which are sorted
+   lexicographically. Array B is treated as case-insensitive.
+
+   If either of the arrays is NULL, the other one is returned unchanged.
+
+   Otherwise, if A[0] begins with a lowercase letter, all items from B
+   will be converted to lowercase.
+
+   Both A and B (but not their elements) are freed prior to returning.
+
+   Note: This function assumes that no string from A is present in B and
+   vice versa.
+ */
+static char **
+compl_concat (char **a, char **b)
+{
+  size_t i, j, k, n = 0, an, bn;
+  char **ret;
+  int lwr = 0;
+
+  if (a)
+    {
+      lwr = mu_islower (a[0][0]);
+      for (an = 0; a[an]; an++)
+	;
+    }
+  else
+    return b;
+
+  if (b)
+    {
+      for (bn = 0; b[bn]; bn++)
+	{
+	  if (lwr)
+	    mu_strlower (b[bn]);
+	}
+    }
+  else
+    return a;
+
+  i = an == 1 ? 0 : 1;
+  j = bn == 1 ? 0 : 1;
+
+  n = (an - i) + (bn - j) + 1;
+  ret = mu_calloc (n + 1, sizeof (ret[0]));
+
+  if (an == bn && an == 1)
+    {
+      /* Compute LCP of both entries */
+      for (i = 0; a[i] && b[i] && a[i] == b[i]; i++)
+	;
+      ret[0] = mu_alloc (i + 1);
+      memcpy (ret[0], a[0], i);
+      ret[0][i] = 0;
+    }
+  else
+    /* The first entry is the LCP of the rest. Select the shortest one. */
+    ret[0] = mu_strdup ((strlen (a[0]) < strlen (b[0])) ? a[0] : b[0]);
+
+  if (i)
+    free (a[0]);
+  if (j)
+    free (b[0]);
+
+  k = 1;
+  while (k < n)
+    {
+      if (!a[i])
+	{
+	  memcpy (ret + k, b + j, sizeof (b[0]) * (bn - j));
+	  break;
+	}
+      else if (!b[j])
+	{
+	  memcpy (ret + k, a + i, sizeof (a[0]) * (an - i));
+	  break;
+	}
+      else
+	ret[k++] = (strcmp (a[i], b[j]) < 0) ? a[i++] : b[j++];
+    }
+  ret[n] = NULL;
+  free (a);
+  free (b);
+  return ret;
+}
+
+static char *header_generator (const char *text, int state);
+
+/* Internal completion generator for commands that take a message list
+   (if MATCHES is NULL), or a messages list followed by another argument
+   (if MATCHES is not NULL).
+
+   In the latter case the MATCHES function generates expansions for the
+   last argument. It is used for such commands as write, where the last
+   object is a mailbox or pipe, where it is a command).
+
+   CLOSURE supplies argument for MATCHES. It is ignored if MATCHES is NULL.
+*/
+static char **
+msglist_closure_compl (int argc, char **argv, int point,
+		       char **(*matches) (void*),
+		       void *closure)
+{
+  char *text = (point & COMPL_WS) ? "" : argv[argc-1];
+  size_t len = strlen (text);
+
+  if (text[0] == ':')
+    {
+      if (text[1] && (text[1] == '/' || text[2]))
+	{
+	  ml_set_completion_append_character (0);
+	  ml_attempted_completion_over ();
+	  return NULL;
+	}
+      ml_set_completion_append_character (' ');
+      return rl_completion_matches (text, msgtype_generator);
+    }
+
+  ml_set_completion_append_character (0);
+  if (len && text[len-1] == ':')
+    {
+      char **ret = mu_calloc(2, sizeof (ret[0]));
+      ret[0] = strcat (strcpy (mu_alloc (len + 2), text), "/");
+      return ret;
+    }
+
+  if (matches && (point & COMPL_LASTARG))
+    return compl_concat (matches (closure),
+			 rl_completion_matches (text, header_generator));
+  else
+    return rl_completion_matches (text, header_generator);
+}
 
 /* Completion functions */
 char **
-no_compl (int argc MU_ARG_UNUSED, char **argv MU_ARG_UNUSED, int ws MU_ARG_UNUSED)
+no_compl (int argc MU_ARG_UNUSED, char **argv MU_ARG_UNUSED,
+	  int flags MU_ARG_UNUSED)
 {
   ml_attempted_completion_over ();
   return NULL;
 }
 
 char **
-msglist_compl (int argc, char **argv, int ws)
+msglist_compl (int argc, char **argv, int point)
 {
-  /* FIXME */
-  ml_attempted_completion_over ();
-  return NULL;
+  return msglist_closure_compl (argc, argv, point, NULL, NULL);
 }
 
 char **
-command_compl (int argc, char **argv, int ws)
+command_compl (int argc, char **argv, int point)
 {
   ml_set_completion_append_character (0);
-  if (ws)
+  if (point & COMPL_WS)
     return NULL;
   return rl_completion_matches (argv[argc-1], ml_command_generator);
 }
-
-/* Generate file list based on reference prefix TEXT, relative to PATH.
-   Remove PATHLEN leading characters from the returned names. Replace
-   them with REPL unless it is 0.
-
-   Select only those files that match given FLAGS (MU_FOLDER_ATTRIBUTE_*
-   constants).
-
-   STATE is 0 for the first call, 1 otherwise.
- */
-static char *
-file_generator (const char *text, int state,
-		char *path, size_t pathlen,
-		char repl,
-		int flags)
+
+struct filegen
 {
-  static mu_list_t list;
-  static mu_iterator_t itr;
+  mu_list_t list;     /* List of matches */
+  mu_iterator_t itr;  /* Iterator over this list */
+  size_t prefix_len;  /* Length of initial prefix */
+  char repl;          /* Character to replace initial prefix with */ 
+  int flags;
+};
 
-  if (!state)
+static void
+filegen_free (struct filegen *fg)
+{
+  mu_iterator_destroy (&fg->itr);
+  mu_list_destroy (&fg->list);
+}
+
+#define PREFIX_AUTO 0
+
+static int
+folder_match_url (mu_folder_t folder, mu_url_t url)
+{
+  mu_url_t furl;
+  int rc = mu_folder_get_url (folder, &furl);
+  if (rc)
     {
-      char *wcard;
-      mu_folder_t folder;
-      size_t count;
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_folder_get_url", NULL, rc);
+      return 0;
+    }
+  return mu_url_is_same_scheme (url, furl)
+	  && mu_url_is_same_user (url, furl)
+	  && mu_url_is_same_host (url, furl)
+	  && mu_url_is_same_portstr (url, furl);
+}
 
-      wcard = mu_alloc (strlen (text) + 2);
-      strcat (strcpy (wcard, text), "*");
+static int
+filegen_init (struct filegen *fg,
+	      const char *text,
+	      const char *folder_path,
+	      int type,
+	      size_t prefix_len,
+	      int repl,
+	      int flags)
+{
+  char *pathref;
+  char *wcard;
+  mu_folder_t folder;
+  size_t count, i, len;
+  mu_url_t url;
+  int rc;
+  int free_folder;
 
-      if (mu_folder_create (&folder, path))
-	{
-	  free (wcard);
-	  return NULL;
-	}
-
-      mu_folder_list (folder, path, wcard, 1, &list);
-      free (wcard);
-      mu_folder_destroy (&folder);
-
-      if (mu_list_count (list, &count) || count == 0)
-	{
-	  mu_list_destroy (&list);
-	  return NULL;
-	}
-      else if (count == 1)
-	ml_set_completion_append_character (0);
-
-      if (mu_list_get_iterator (list, &itr))
-	{
-	  mu_list_destroy (&list);
-	  return NULL;
-	}
-      mu_iterator_first (itr);
+  rc = mu_url_create (&url, folder_path);
+  if (rc)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_url_create", folder_path, rc);
+      return -1;
     }
 
-  while (!mu_iterator_is_done (itr))
+  rc = mu_mailbox_get_folder (mbox, &folder);
+  if (rc == 0)
+    {
+      if (folder_match_url (folder, url))
+	free_folder = 0;
+      else
+	folder = NULL;
+    }
+  else
+    folder = NULL;
+
+  if (!folder)
+    {
+      if (util_get_folder (&folder, url, type))
+	return -1;
+      free_folder = 1;
+    }
+
+  pathref = mu_strdup (text);
+  len = strlen (pathref);
+  for (i = len; i > 0; i--)
+    if (pathref[i-1] == '/')
+      break;
+  wcard = mu_alloc (len - i + 2);
+  strcpy (wcard, pathref + i);
+  strcat (wcard, "%");
+  pathref[i] = 0;
+
+  fg->prefix_len = prefix_len;
+
+  mu_folder_list (folder, pathref, wcard, 1, &fg->list);
+  free (wcard);
+  free (pathref);
+  if (free_folder)
+    mu_folder_destroy (&folder);
+
+  if (mu_list_count (fg->list, &count) || count == 0)
+    {
+      mu_list_destroy (&fg->list);
+      return -1;
+    }
+  else if (count == 1)
+    {
+      ml_set_completion_append_character (0);
+      if (flags & MU_FOLDER_ATTRIBUTE_DIRECTORY)
+	{
+	  struct mu_list_response *resp;
+	  mu_list_head (fg->list, (void**)&resp);
+	  if ((resp->type & MU_FOLDER_ATTRIBUTE_DIRECTORY)
+	      && strcmp (resp->name, text) == 0)
+	    ml_set_completion_append_character (resp->separator);
+	}
+    }
+
+  if (mu_list_get_iterator (fg->list, &fg->itr))
+    {
+      mu_list_destroy (&fg->list);
+      return -1;
+    }
+  mu_iterator_first (fg->itr);
+  fg->repl = repl;
+  fg->flags = flags;
+  return 0;
+}
+
+static char *
+filegen_next (struct filegen *fg)
+{
+  while (!mu_iterator_is_done (fg->itr))
     {
       struct mu_list_response *resp;
-      mu_iterator_current (itr, (void**)&resp);
-      mu_iterator_next (itr);
-      if (resp->type & flags)
+      mu_iterator_current (fg->itr, (void**)&resp);
+      mu_iterator_next (fg->itr);
+      if (resp->type & fg->flags)
 	{
 	  char *ret;
-	  if (pathlen)
-	    {
-	      size_t len = strlen (resp->name + pathlen);
-	      char *ptr;
+	  char *name;
+	  char *ptr;
 
-	      ret = mu_alloc (len + (repl ? 1 : 0) + 1);
-	      ptr = ret;
-	      if (repl)
-		*ptr++ = repl;
-	      memcpy (ptr, resp->name + pathlen, len);
-	      ptr[len] = 0;
-	    }
-	  else
-	    ret = mu_strdup (resp->name);
+	  name = resp->name + fg->prefix_len;
+	  ret = mu_alloc (strlen (name) + (fg->repl ? 1 : 0) + 1);
+	  ptr = ret;
+	  if (fg->repl)
+	    *ptr++ = fg->repl;
+	  strcpy (ptr, name);
 	  return ret;
 	}
     }
-  mu_iterator_destroy (&itr);
-  mu_list_destroy (&list);
+  filegen_free (fg);
   return NULL;
 }
 
 static char *
 folder_generator (const char *text, int state)
 {
-  char *ret;
-  static size_t pathlen;
+  static struct filegen fg;
 
   if (!state)
     {
-      char *path = util_folder_path ("");
+      int rc;
+      char *path = util_folder_path ("+");
       if (!path)
 	return NULL;
 
-      pathlen = strlen (path);
-      ret = file_generator (text, state, path, pathlen, '+',
-			    MU_FOLDER_ATTRIBUTE_ALL);
+      rc = filegen_init (&fg, text, path,
+			 any_folder,
+			 PREFIX_AUTO, '+',
+			 MU_FOLDER_ATTRIBUTE_ALL);
       free (path);
-    }
-  else
-    ret = file_generator (text, state, NULL, pathlen, '+',
-			  MU_FOLDER_ATTRIBUTE_ALL);
-  return ret;
-}
-
-static char *
-msgtype_generator (const char *text, int state)
-{
-  static char types[] = "dnorTtu";
-  static int i;
-  char *s;
-
-  if (!state)
-    {
-      if (text[1])
+      if (rc)
 	return NULL;
-      i = 0;
     }
-  if (!types[i])
-    return NULL;
-  s = malloc (2);
-  if (s)
-    {
-      s[0] = types[i++];
-      s[1] = 0;
-    }
-  return s;
+  return filegen_next (&fg);
 }
 
 static char *
@@ -481,6 +693,7 @@ header_generator (const char *text, int state)
     MU_HEADER_DELIVERY_DATE,
     MU_HEADER_ENVELOPE_TO,
     MU_HEADER_X_EXPIRE_TIMESTAMP,
+    MU_HEADER_USER_AGENT,
     NULL
   };
 
@@ -494,18 +707,18 @@ header_generator (const char *text, int state)
     {
       i++;
       if (mu_c_strncasecmp (hdr, text, len) == 0)
-	return mu_strdup (hdr);
+	return strcat (strcpy (mu_alloc (strlen (hdr) + 3), hdr), ":/");
     }
 
   return NULL;
 }
 
-static char **
-file_compl_internal (int argc, char **argv, int ws, int msglist)
+char **
+file_compl (int argc, char **argv, int point)
 {
   char *text;
 
-  if (ws)
+  if (point & COMPL_WS)
     {
       ml_set_completion_append_character (0);
       ml_attempted_completion_over ();
@@ -514,113 +727,133 @@ file_compl_internal (int argc, char **argv, int ws, int msglist)
 
   text = argv[argc-1];
 
-  if (msglist)
-    {
-      if (text[0] == ':')
-	{
-	  ml_set_completion_append_character (' ');
-	  return rl_completion_matches (text, msgtype_generator);
-	}
-      else if (mu_isalpha (text[0]))
-	{
-	  ml_set_completion_append_character (':');
-	  return rl_completion_matches (text, header_generator);
-	}
-      else if (mu_isdigit (text[0]))
-	{
-	  ml_attempted_completion_over ();
-	  return NULL;
-	}
-    }
-
   switch (text[0])
     {
     case '+':
-      text++;
-      break;
+      return rl_completion_matches (text + 1, folder_generator);
 
     case '%':
     case '#':
     case '&':
       ml_attempted_completion_over ();
-      return NULL;
+      break;
 
     default:
-      return NULL; /* Will be expanded by readline itself */
+      /* Suppose it is a file name */
+      return rl_completion_matches (text, rl_filename_completion_function);
     }
 
-  return rl_completion_matches (text, folder_generator);
+  return NULL;
+}
+
+struct compl_closure
+{
+  int argc;
+  char **argv;
+  int point;
+};
+
+static char **
+file_compl_matches (void *closure)
+{
+  struct compl_closure *cp = closure;
+  return file_compl (cp->argc, cp->argv, cp->point);
 }
 
 char **
-file_compl (int argc, char **argv, int ws)
+msglist_file_compl (int argc, char **argv, int point)
 {
-  return file_compl_internal (argc, argv, ws, 0);
-}
-
-char **
-msglist_file_compl (int argc, char **argv, int ws)
-{
-  return file_compl_internal (argc, argv, ws, 1);
+  struct compl_closure clos = { argc, argv, point };
+  return msglist_closure_compl (argc, argv, point, file_compl_matches, &clos);
 }
 
 static char *
 dir_generator (const char *text, int state)
 {
-  char *ret;
-  static size_t pathlen;
-  static int repl;
+  static struct filegen fg;
 
   if (!state)
     {
       char *path;
+      char *p;
+      int rc;
+      char repl;
+      size_t prefix_len;
+
       switch (text[0])
 	{
 	case '+':
-	  text++;
-	  repl = '+';
-	  path = util_folder_path (text);
-	  pathlen = strlen (path) - strlen (text);
+	  {
+	    char *f;
+	    repl = '+';
+	    f = util_folder_path ("+");
+	    prefix_len = strlen (f);
+	    path = mu_make_file_name (f, text + 1);
+	    free (f);
+	  }
 	  break;
 
 	case '~':
 	  repl = '~';
 	  if (text[1] == '/')
 	    {
-	      path = mu_get_homedir ();
-	      text += 2;
-	      pathlen = strlen (path);
-	      break;
+	      char *home = mu_get_homedir ();
+	      prefix_len = strlen (home);
+	      path = mu_make_file_name (home, text + 2);
+	      free (home);
 	    }
-	  /* else FIXME! */
+	  else
+	    {
+	      ml_attempted_completion_over ();
+	      return NULL;
+	      /* FIXME: implement user-name completion */
+	    }
+	  break;
 
 	case '/':
 	  path = mu_strdup (text);
-	  pathlen = 0;
+	  prefix_len = 0;
 	  repl = 0;
 	  break;
 
 	default:
-	  path = mu_strdup ("./");
-	  pathlen = 2;
+	  {
+	    char *cwd = mu_getcwd ();
+	    prefix_len = strlen (cwd);
+	    path = mu_make_file_name (cwd, text);
+	    free (cwd);
+	  }
 	  repl = 0;
 	}
-
-      ret = file_generator (text, state, path, pathlen, repl,
-			    MU_FOLDER_ATTRIBUTE_DIRECTORY);
-      free (path);
+      p = strrchr (path, '/');
+      if (*p)
+	{
+	  if (p[1])
+	    *p++ = 0;
+	  else
+	    p = "";
+	  rc = filegen_init (&fg, p, path[0] ? path : "/",
+			     local_folder,
+			     prefix_len, repl,
+			     MU_FOLDER_ATTRIBUTE_DIRECTORY);
+	}
+      else
+	{
+	  ml_attempted_completion_over ();
+	  rc = -1;
+	}
+      if (rc)
+	return NULL;
     }
-  else
-    ret = file_generator (text, state, NULL, pathlen, repl,
-			  MU_FOLDER_ATTRIBUTE_DIRECTORY);
-  return ret;
+
+  return filegen_next (&fg);
 }
 
 char **
-dir_compl (int argc, char **argv, int ws)
+dir_compl (int argc, char **argv, int point)
 {
   ml_attempted_completion_over ();
-  if (ws)
+  if (point & COMPL_WS)
     {
       ml_set_completion_append_character (0);
       return NULL;
@@ -648,24 +881,11 @@ alias_generator (const char *text, int state)
 }
 
 char **
-alias_compl (int argc, char **argv, int ws)
+alias_compl (int argc, char **argv, int point)
 {
   ml_attempted_completion_over ();
-  return rl_completion_matches (ws ? "" : argv[argc-1], alias_generator);
-}
-
-static char *
-mkfilename (const char *dir, const char *file)
-{
-  size_t len = strlen (dir) + 1 + strlen (file) + 1;
-  char *p = malloc (len);
-  if (p)
-    {
-      strcpy (p, dir);
-      strcat (p, "/");
-      strcat (p, file);
-    }
-  return p;
+  return rl_completion_matches ((point & COMPL_WS) ? "" : argv[argc-1],
+				alias_generator);
 }
 
 static char *
@@ -731,7 +951,7 @@ exec_generator (const char *text, int state)
 
       while ((ent = readdir (dp)))
 	{
-	  char *name = mkfilename (dir, ent->d_name);
+	  char *name = mu_make_file_name (dir, ent->d_name);
 	  if (name)
 	    {
 	      int rc = access (name, X_OK);
@@ -757,10 +977,26 @@ exec_generator (const char *text, int state)
   return NULL;
 }
 
-char **
-exec_compl (int argc, char **argv, int ws)
+static char **
+exec_matches (void *closure)
 {
-  return rl_completion_matches (ws ? "" : argv[argc-1], exec_generator);
+  return rl_completion_matches ((char *)closure, exec_generator);
+}
+
+char **
+exec_compl (int argc, char **argv, int point)
+{
+  return msglist_closure_compl (argc, argv, point,
+				exec_matches,
+				(point & COMPL_WS) ? "" : argv[argc-1]);
+}
+
+char **
+shell_compl (int argc, char **argv, int point)
+{
+  if (argc == 2)
+    return rl_completion_matches (argv[1], exec_generator);
+  return no_compl (argc, argv, point);
 }
 
 #else
@@ -780,7 +1016,7 @@ static int ch_kill;
 static struct termios term_settings;
 
 int
-set_tty ()
+set_tty (void)
 {
   struct termios new_settings;
 
@@ -804,7 +1040,7 @@ set_tty ()
 }
 
 void
-restore_tty ()
+restore_tty (void)
 {
   tcsetattr (STDOUT, TCSADRAIN, &term_settings);
 }
@@ -815,7 +1051,7 @@ restore_tty ()
 static struct termio term_settings;
 
 int
-set_tty ()
+set_tty (void)
 {
   struct termio new_settings;
 
@@ -835,7 +1071,7 @@ set_tty ()
 }
 
 void
-restore_tty ()
+restore_tty (void)
 {
   ioctl(STDOUT, TCSETA, &term_settings);
 }
@@ -846,7 +1082,7 @@ restore_tty ()
 static struct sgttyb term_settings;
 
 int
-set_tty ()
+set_tty (void)
 {
   struct sgttyb new_settings;
 
@@ -864,7 +1100,7 @@ set_tty ()
 }
 
 void
-restore_tty ()
+restore_tty (void)
 {
   ioctl(STDOUT, TIOCSETP, &term_settings);
 }
@@ -1059,7 +1295,7 @@ ml_set_completion_append_character (int c MU_ARG_UNUSED)
 }
 
 void
-ml_attempted_completion_over ()
+ml_attempted_completion_over (void)
 {
 }
 
